@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 #
-# $Id: test_autodock4.py,v 1.26 2009/09/04 21:02:39 rhuey Exp $
+# $Id: test_autodock4.py,v 1.29 2009/12/08 18:06:20 rhuey Exp $
 #
 
 """
@@ -168,6 +168,48 @@ class AutoDock_simple_test( unittest.TestCase ):
         self.assertEqual( self.computed, self.expected_outcome )
 #______________________________________________________________________________
 
+class AutoDock4_1pgp_no_extension( AutoDock_simple_test ):
+    """Test that autodock4 stops early if .dpf extension is missing
+    keywords are specified."""
+    dpf_stem = "1pgp_no_extension"
+    print "in 1pgp_no_extension"
+    expected_outcome = False # True means Successful Completion!
+    def setUp( self ):
+        """Set up for autodock4 tests. Locate the autodock binary now during setUp."""
+        self.dlg_filename = "test_" + self.dpf_stem + ".dlg"
+        self.computed = run_AutoDock( self.dpf_stem , self.dlg_filename )
+
+#______________________________________________________________________________
+
+class AutoDock4_1pgp_wrong_extension( AutoDock_simple_test ):
+    """Test that autodock4 stops early if extension is not '.dpf'
+    keywords are specified."""
+    dpf_stem = "1pgp.fpd"
+    print "in 1pgp_wrong_extension"
+    expected_outcome = False # True means Successful Completion!
+    def setUp( self ):
+        """Set up for autodock4 tests. Locate the autodock binary now during setUp."""
+        self.dlg_filename = "test_" + self.dpf_stem + ".dlg"
+        self.computed = run_AutoDock( self.dpf_stem , self.dlg_filename )
+
+#______________________________________________________________________________
+
+
+class AutoDock4_1pgp_two_extensions( AutoDock_simple_test ):
+    """Test that autodock4 stops early if dpf name includes two .dpf
+    keywords are specified."""
+    dpf_stem = "1pgp.dpf"
+    print "in 1pgp_two_extensions"
+    expected_outcome = False # True means Successful Completion!
+    def setUp( self ):
+        """Set up for autodock4 tests. Locate the autodock binary now during setUp."""
+        self.dlg_filename = "test_" + self.dpf_stem + ".dlg"
+        self.computed = run_AutoDock( self.dpf_stem +'.dpf', self.dlg_filename )
+
+
+#______________________________________________________________________________
+
+
 class AutoDock4_1pgp_ligand_types_map_mismatch( AutoDock_simple_test ):
     """Test that autodock4 stops early if number of maps do not equal number
     of ligand types"""
@@ -246,6 +288,12 @@ class AutoDock4_1pgp_unbound_model_illegal_test( AutoDock_simple_test ):
     in dpf """
     dpf_stem = "1pgp_unbound_model_illegal"
     expected_outcome = False # True means Successful Completion!
+#______________________________________________________________________________
+
+class AutoDock4_1pgp_ga_select_tournament_test( AutoDock_simple_test ):
+    """Test that autodock 4.1 works when ga_select_tournament is set in the DPF."""
+    dpf_stem = "1pgp_ga_select_tournament"
+    expected_outcome = True # True means Successful Completion!
 
 #______________________________________________________________________________
 
@@ -277,6 +325,12 @@ class AutoDock4_1pgp_test( AutoDock_test ):
     expected_outcome = True # True means Successful Completion!
 #______________________________________________________________________________
 
+class AutoDock4_1pgp_smaller_test( AutoDock_test ):
+    """Test that autodock4 executes using fewer parameters and an extremely short run."""
+    dpf_stem = "1pgp_smaller"
+    expected_outcome = True # True means Successful Completion!
+#______________________________________________________________________________
+
 class AutoDock4_1pgp_no_parameter_file_test( AutoDock_test ):
     """Test that autodock4 works using default parameter library."""
     dpf_stem = "1pgp_no_parameter_file"
@@ -367,6 +421,10 @@ if __name__ == '__main__':
     #  or conveniently comment out tests we're not interested in.
     #  NOTE:  Remember to add new TestCase class names to the list "test_cases"
     test_cases = [
+        # tests for .dpf extension:
+        'AutoDock4_1pgp_no_extension',
+        'AutoDock4_1pgp_wrong_extension',
+        'AutoDock4_1pgp_two_extensions',
         # simple tests:
         'AutoDock4_1pgp_ligand_types_map_mismatch',
         'AutoDock4_1pgp_illegal_keyword_test',
@@ -380,8 +438,10 @@ if __name__ == '__main__':
         'AutoDock4_1pgp_two_mapsets_test',
         'AutoDock4_1pgp_unbound_set_illegal_test',
         'AutoDock4_1pgp_unbound_model_illegal_test', #1
+        'AutoDock4_1pgp_ga_select_tournament_test',
         ## tests which check for specific value
         'AutoDock4_1pgp_test',
+        'AutoDock4_1pgp_smaller_test',
         'AutoDock4_1pgp_no_parameter_file_test',
         ## tests for unbound values 
         'AutoDock4_1pgp_unbound_default_test',
