@@ -1,6 +1,6 @@
 /*
 
- $Id: call_glss.cc,v 1.40 2009/05/08 23:02:11 rhuey Exp $
+ $Id: call_glss.cc,v 1.46 2010/01/08 20:13:47 mp Exp $
 
  AutoDock  
 
@@ -377,6 +377,62 @@ State call_glss(Global_Search *global_method, Local_Search *local_method,
 
     (void)fprintf( logFile, "Beginning Lamarckian Genetic Algorithm (LGA), with a maximum of %u\nenergy evaluations.\n\n", num_evals);
 
+ // M Pique 28 Oct 2009 - adding a (harmless...) thisPop.msort(1) here
+ // broke the unbound-extended test. Investigating why
+ // Conclusion: the test is sensitive to population order since it does
+ // only one generation of glss.  So I'm removing the msort(1) and doing
+ // the search for lowest-energy individual by hand.
+#ifdef DEBUGMSORT
+     double *beforesort = new double[pop_size];
+     double *aftersort = new double[pop_size];
+     int *beforeorder = new int[pop_size];
+     int *afterorder = new int[pop_size];
+
+    (void)fprintf(logFile,"@@ #evals before msort %-6u  DEBUGMSORT call_glss.cc\n", evaluate.evals());
+     for (i=0; i<pop_size; i++) beforesort[i] = thisPop[i].value(Normal_Eval);
+     //thisPop.msort(1);
+     for (i=0; i<pop_size; i++) aftersort[i] = thisPop[i].value(Normal_Eval);
+
+     // set beforeorder[i] to location j of each aftersort value
+     for (i=0; i<pop_size; i++) beforeorder[i] = -1 ; // flag unset values
+     for (i=0; i<pop_size; i++) \
+     for (j=0;j<pop_size && aftersort[i]!=beforesort[beforeorder[i]=j];j++) ;
+     // set afterorder[i] to location j of each beforesort value
+     for (i=0; i<pop_size; i++) afterorder[i] = -1 ; // flag unset values
+     for (i=0; i<pop_size; i++) \
+     for (j=0;j<pop_size && beforesort[i]!=aftersort[afterorder[i]=j];j++) ;
+     // report 
+    (void)fprintf(logFile,"@@ #evals after msort %-6u  DEBUGMSORT call_glss.cc\n", evaluate.evals());
+    (void)fprintf(logFile,"@@ pop before     after sort   locn   DEBUGMSORT call_glss.cc\n");
+     for (i=0; i<pop_size; i++) 
+       (void)fprintf(logFile,"%3d %9.2f %3d   %9.2f %3d\n", 
+         i, beforesort[i], afterorder[i], aftersort[i], beforeorder[i]);
+
+      delete[] beforesort;
+      delete[] aftersort;
+      delete[] beforeorder;
+      delete[] afterorder;
+#endif
+    if(pop_size>0) {
+       double bestenergy = thisPop[0].value(Normal_Eval);
+       for (i=1; i<pop_size; i++) if(bestenergy>thisPop[i].value(Normal_Eval)) \
+         bestenergy=thisPop[i].value(Normal_Eval);
+       (void)fprintf(logFile,"Initial-Value: %.3f\n", bestenergy);
+
+       // print "Population at Generation:" line with low/high/mean/median/stddev...
+       // and search counts (expected to be zero)
+       if(outlev>0) {
+         (void) thisPop.printPopulationStatisticsVerbose(logFile, 
+         num_generations, evaluate.evals(), "");
+	 fprintf(logFile, " cg_count: %u", global_method->cg_count);
+	 fprintf(logFile, " ci_count: %u", global_method->ci_count);
+	 fprintf(logFile, " mg_count: %u", global_method->mg_count);
+	 fprintf(logFile, " mi_count: %u", global_method->mi_count);
+	 fprintf(logFile, " ls_count: %u", local_method->count);
+	 fprintf(logFile, "\n");
+	 }
+     }
+
     do {
         ++num_generations;
 
@@ -415,6 +471,28 @@ State call_glss(Global_Search *global_method, Local_Search *local_method,
             (void)fprintf( logFile, "</generation>\n\n\n");
         }
 
+       // Print extended generational statistics 
+       //  every generation 1 to 20
+       //  every tenth generation 20 to extOutputEveryNgens (100)
+       //  every "extOutputEveryNgens" (100) if greater than that
+       // This is purely for studying population convergence -  M Pique 2010
+       if (outlev > 1 && extOutputEveryNgens != 0 && (
+	 (num_generations <= 20) ||
+	 (num_generations <= extOutputEveryNgens && num_generations%10 == 0) ||
+         (num_generations%extOutputEveryNgens == 0 )
+	 )) {
+	       // print "Population at Generation:" line with low/high/mean/median/stddev...
+	       // followed by global search stats (crossover count, mutation count)
+	       // and local search stats (invocation count)
+	       (void) thisPop.printPopulationStatisticsVerbose(logFile, 
+		 num_generations, evaluate.evals(), "");
+		 fprintf(logFile, " cg_count: %u", global_method->cg_count);
+		 fprintf(logFile, " ci_count: %u", global_method->ci_count);
+		 fprintf(logFile, " mg_count: %u", global_method->mg_count);
+		 fprintf(logFile, " mi_count: %u", global_method->mi_count);
+		 fprintf(logFile, " ls_count: %u", local_method->count);
+		 fprintf(logFile, "\n");
+		 }
         if (pop_size > 1 && outlev > 3) { minmeanmax( logFile, thisPop, num_generations, info ); }
 
         if (strcmp (FN_pop_file, "") != 0) { // YES, do print!
@@ -431,5 +509,18 @@ State call_glss(Global_Search *global_method, Local_Search *local_method,
 
     thisPop.msort(1);
     (void)fprintf(logFile,"Final-Value: %.3f\n", thisPop[0].value(Normal_Eval));
+
+    // print "Population at Generation:" line with low/high/mean/median/stddev...
+    if(outlev>0) {
+       (void) thisPop.printPopulationStatisticsVerbose(logFile, 
+        num_generations, evaluate.evals(), "");
+	 fprintf(logFile, " cg_count: %u", global_method->cg_count);
+	 fprintf(logFile, " ci_count: %u", global_method->ci_count);
+	 fprintf(logFile, " mg_count: %u", global_method->mg_count);
+	 fprintf(logFile, " mi_count: %u", global_method->mi_count);
+	 fprintf(logFile, " ls_count: %u", local_method->count);
+	 fprintf(logFile, "\n");
+	 }
+
     return( thisPop[0].state(sInit.ntor) );
 }
