@@ -1,6 +1,6 @@
 /*
 
- $Id: constants.h,v 1.23 2009/05/08 23:02:12 rhuey Exp $
+ $Id: constants.h,v 1.28 2009/12/16 23:35:54 rhuey Exp $
 
  AutoDock 
 
@@ -225,9 +225,9 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 
 #define NROWS 16
 
-/* Output frequency in GAs */
+/* Statistics output frequency in GAs */
  
-#define OUTLEV0_GENS 30000 /* output every 30,000 generations */
+#define OUTLEV0_GENS 0   /* Never output based on number of generations */
 #define OUTLEV1_GENS 100 /* output every 100 generations */
 #define OUTLEV2_GENS 1 /* output every generation */
 
@@ -427,10 +427,6 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
  * "DOCKED: "  gives DOCKED prefix.
  * Note: no new line at end of some of these formats:
  */
-#define FORMAT_PDBQ_ATOM_RESSTR         "%sATOM  %5d  %.13s    %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f"
-#define FORMAT_PDBQ_ATOM_RESNUM         "%sATOM  %5d  %.8s%5d    %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f"
-#define FORMAT_PDBQ_ATOM_RANKRUN_STR      "ATOM  %5d  %.13s    %8.3f%8.3f%8.3f%6d%6d    %+6.2f %8.3f\n"
-#define FORMAT_PDBQ_ATOM_RUN_NUM          "ATOM  %5d  %.8s%5d    %8.3f%8.3f%8.3f%6d%+6.2f    %6.3f\n"
 
 /* 
  * PDBQT format
@@ -458,10 +454,10 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 // |     |     |   ||   ||   |   |       |       |       |     |         |      |                      
 // 65432154321 43211321 143211   876543218765432187654321654321654321    654321 21
 // ATOM     10  HO4 PGP     1      22.065  29.222  38.002  1.00  0.00     0.210 HD
-#define FORMAT_PDBQT_ATOM_RESSTR         "%sATOM  %5d  %.13s    %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f %-2s"
-#define FORMAT_PDBQT_ATOM_RESNUM         "%sATOM  %5d  %.8s%5d    %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f %-2s"
-#define FORMAT_PDBQT_ATOM_RANKRUN_STR      "ATOM  %5d  %.13s    %8.3f%8.3f%8.3f%6d%6d    %+6.2f %8.3f %-2s\n"
-#define FORMAT_PDBQT_ATOM_RUN_NUM          "ATOM  %5d  %.8s%5d    %8.3f%8.3f%8.3f%6d%+6.2f    %6.3f %-2s\n"
+#define FORMAT_PDBQT_ATOM_RESSTR         "%sATOM  %5d %.13s     %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f %-2s"
+#define FORMAT_PDBQT_ATOM_RESNUM         "%sATOM  %5d %.8s%5d     %8.3f%8.3f%8.3f%+6.2f%+6.2f    %+6.3f %-2s"
+#define FORMAT_PDBQT_ATOM_RANKRUN_STR      "ATOM  %5d %.13s     %8.3f%8.3f%8.3f%6d%6d    %+6.2f %8.3f %-2s\n"
+#define FORMAT_PDBQT_ATOM_RUN_NUM          "ATOM  %5d %.8s%5d     %8.3f%8.3f%8.3f%6d%+6.2f    %6.3f %-2s\n"
                                                                   
 #endif /* _PDB_FORMATS */
 
@@ -533,6 +529,12 @@ const Real TK = 298.15;      // Room temperature, in K
 enum Unbound_Model { Unbound_Default=0, Unbound_Same_As_Bound=1, Extended=2, Compact=3, User=4 };
 #endif
 
+/*
+Constants used for the PSO Work
+ */
+#define D_max      128   // Max number of dimensions of the search space
+#define S_max 	  1024	 // Max swarm size
+#define R_max 	  1024   // Max number of runs
 /*----------------------------------------------------------------------------* 
  * End of file                                                                * 
  *----------------------------------------------------------------------------*/
